/*
 * @(#)des_crypt.h 1.4 88/02/08 (C) 1986 SMI
 *
 * des_crypt.h, des library routine interface
 * Copyright (C) 1986, Sun Microsystems, Inc.
 */
#ifndef __DES_CRYPTH
#define __DES_CRYPTH

#include <c_varieties.h>

#define DES_MAXDATA 8192	/* max bytes encrypted in one call */
#define DES_DIRMASK (1 << 0)
#define DES_ENCRYPT (0*DES_DIRMASK)	/* Encrypt */
#define DES_DECRYPT (1*DES_DIRMASK)	/* Decrypt */


#define DES_DEVMASK (1 << 1)
#define	DES_HW (0*DES_DEVMASK)	/* Use hardware device */ 
#define DES_SW (1*DES_DEVMASK)	/* Use software device */


#define DESERR_NONE 0	/* succeeded */
#define DESERR_NOHWDEVICE 1	/* succeeded, but hw device not available */
#define DESERR_HWERROR 2	/* failed, hardware/driver error */
#define DESERR_BADPARAM 3	/* failed, bad parameter to call */

#define DES_FAILED(err) \
	((err) > DESERR_NOHWDEVICE)

/*
 * cbc_crypt()
 * ecb_crypt()
 *
 * Encrypt (or decrypt) len bytes of a buffer buf.
 * The length must be a multiple of eight.
 * The key should have odd parity in the low bit of each byte.
 * ivec is the input vector, and is updated to the new one (cbc only).
 * The mode is created by oring together the appropriate parameters.
 * DESERR_NOHWDEVICE is returned if DES_HW was specified but
 * there was no hardware to do it on (the data will still be
 * encrypted though, in software).
 */


/*
 * Cipher Block Chaining mode
 */
EXTERN_FUNCTION ( extern int cbc_crypt, (char *, char *, unsigned, unsigned, char *) ); 
/*
	char *key;	
	char *buf;
	unsigned len;
	unsigned mode;
	char *ivec;	
*/ 


/*
 * Electronic Code Book mode
 */
EXTERN_FUNCTION ( extern int ecb_crypt, (char *, char *, unsigned, unsigned) ); 
/*
	char *key;	
	char *buf;
	unsigned len;
	unsigned mode;
*/


/* 
 * Turn password into DES key 
 */ 
EXTERN_FUNCTION ( extern int passwd2des, (char*, char*)); 
/*
        char *pw; 
        char *key;
*/ 


/*
 * Encrypt a secret key given passwd
 * The secret key is passed and returned in hex notation.
 * Its length must be a multiple of 16 hex digits (64 bits).
 */
EXTERN_FUNCTION ( extern int xencrypt,	(char*, char*));
/*
        char *secret;
        char *passwd;
*/


/*
 * Decrypt secret key using passwd
 * The secret key is passed and returned in hex notation.
 * Once again, the length is a multiple of 16 hex digits
 */
EXTERN_FUNCTION ( extern int	xdecrypt, (char*, char*));
/* 
        char *secret; 
        char *passwd; 
*/


#ifndef KERNEL
/* 
 * Set des parity for a key.
 * DES parity is odd and in the low bit of each byte
 */
EXTERN_FUNCTION ( extern void des_setparity, (char *) ); 
/*
	char *key;	
*/
#endif

#endif

