/*	@(#)netdb.h 1.11 88/08/19 SMI from UCB 5.9 4/5/88	*/
/*
 * Copyright (c) 1980,1983,1988 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted
 * provided that this notice is preserved and that due credit is given
 * to the University of California at Berkeley. The name of the University
 * may not be used to endorse or promote products derived from this
 * software without specific prior written permission. This software
 * is provided ``as is'' without express or implied warranty.
 */
 
/*
 * Structures returned by network data base library.
 * All addresses are supplied in host order, and
 * returned in network order (suitable for use in system calls).
 */

#ifndef __NETDB_H
#define __NETDB_H

#include <c_varieties.h>
#include <sys/types.h>

/*
 * Structures returned by network
 * data base library.  All addresses
 * are supplied in host order, and
 * returned in network order (suitable
 * for use in system calls).
 *
 */
struct	hostent {
	char	*h_name;	/* official name of host */
	char	**h_aliases;	/* alias list */
	int	h_addrtype;	/* host address type */
	int	h_length;	/* length of address */
	char	**h_addr_list;	/* list of addresses from name server */
#define	h_addr	h_addr_list[0]	/* address, for backward compatiblity */
};

/*
 * Assumption here is that a network number
 * fits in 32 bits -- probably a poor one.
 */
struct	netent {
	char		*n_name;	/* official name of net */
	char		**n_aliases;	/* alias list */
	int		n_addrtype;	/* net address type */
	unsigned long	n_net;		/* network # */
};

struct	servent {
	char	*s_name;	/* official service name */
	char	**s_aliases;	/* alias list */
	int	s_port;		/* port # */
	char	*s_proto;	/* protocol to use */
};

struct	protoent {
	char	*p_name;	/* official protocol name */
	char	**p_aliases;	/* alias list */
	int	p_proto;	/* protocol # */
};

struct rpcent {
	char	*r_name;	/* name of server for this rpc program */
	char	**r_aliases;	/* alias list */
	int	r_number;	/* rpc program number */
};

EXTERN_FUNCTION ( struct hostent	*gethostbyname, (char *) );
EXTERN_FUNCTION ( struct hostent	*gethostbyaddr, (char *, int, int) );
EXTERN_FUNCTION ( struct hostent	*gethostent, (_VOID_) );

EXTERN_FUNCTION ( struct netent	*getnetbyname, (char *) );
EXTERN_FUNCTION ( struct netent	*getnetbyaddr, (long, int) );
EXTERN_FUNCTION ( struct netent	*getnetent, (_VOID_) );

EXTERN_FUNCTION ( struct servent	*getservbyname, (char *, char *) );
EXTERN_FUNCTION ( struct servent	*getservbyport, (int, char *) );
EXTERN_FUNCTION ( struct servent	*getservent, (_VOID_) );

EXTERN_FUNCTION ( struct protoent	*getprotobyname, (char *) );
EXTERN_FUNCTION ( struct protoent	*getprotobynumber, (int) );
EXTERN_FUNCTION ( struct protoent	*getprotoent, (_VOID_) );

EXTERN_FUNCTION ( struct rpcent	*getrpcbyname, (char *) );
EXTERN_FUNCTION ( struct rpcent	*getrpcbynumber, (int) );
EXTERN_FUNCTION ( struct rpcent	*getrpcent, (_VOID_) );

/*
 * Error return codes from gethostbyname() and gethostbyaddr()
 * (when using the resolver)
 */

/* 
 * Additional decls to please C++.
 */
EXTERN_FUNCTION ( int		sethostent,	(int) );
EXTERN_FUNCTION ( int		endhostent,	() );
EXTERN_FUNCTION ( int		setnetent,	(int) );
EXTERN_FUNCTION ( int		endnetent,	() );
EXTERN_FUNCTION ( int		setservent,	(int) );
EXTERN_FUNCTION ( int		endservent,	() );
EXTERN_FUNCTION ( int		setprotoent,	(int) );
EXTERN_FUNCTION ( int		endprotoent,	() );
EXTERN_FUNCTION ( int 		setrpcent,	(int) );
EXTERN_FUNCTION ( int 		endrpcent,	() );
EXTERN_FUNCTION ( extern int	rcmd,	(char**, u_short, char*, char*, char*, int*));
EXTERN_FUNCTION ( extern int	rexec,	(char**, int, char*, char*, char*, int*));
EXTERN_FUNCTION ( extern int	rresvport,	(int*));
EXTERN_FUNCTION ( extern int	ruserok, (const char*, int, const char*, const char*));
EXTERN_FUNCTION ( extern int 	getnetgrent, (char**, char **, char **));
EXTERN_FUNCTION ( extern void 	setnetgrent, (char*));
EXTERN_FUNCTION ( extern void 	endnetgrent, (_VOID_));

extern  int h_errno;	

#define	HOST_NOT_FOUND	1 /* Authoritive Answer Host not found */
#define	TRY_AGAIN	2 /* Non-Authoritive Host not found, or SERVERFAIL */
#define	NO_RECOVERY	3 /* Non recoverable errors, FORMERR, REFUSED, NOTIMP */
#define	NO_DATA		4 /* Valid name, no data record of requested type */
#define	NO_ADDRESS	NO_DATA		/* no address, look for MX record */

#endif
