/* @(#)cg12_var.h	1.9 of 6/28/90 SMI */

/* Copyright 1990 by Sun Microsystems, Inc. */

/* Sun Color Graphics board 12 (CG12) */

#ifndef cg12_var_DEFINED
#define	cg12_var_DEFINED

#include <sys/types.h>
#include <sys/ioctl.h>
#include <sun/fbio.h>
#include <pixrect/pixrect.h>
#include <sbusdev/cg12reg.h>
#include <pixrect/memvar.h>
#include <sunwindow/cms.h>		/* colormapseg */

#define	CG12_NFBS		6	/* number of frame buffers in a CG12 */

/* description of single CG12 frame buffer */

struct cg12fb
{
    short               group;		/* plane group implemented */
    short               depth;		/* depth, bits */
    struct mprp_data    mprp;		/* memory pixrect data */
};

/*  CG12  Pixrect private data */

struct cg12_data
{
    /* first make it look like a gp device */

    struct cg2fb       *cgpr_va;	/* backward source compatible */
    caddr_t             gp_shmem;	/* pointer to shared memory */
    int                 cgpr_fd;	/* primary flag */
    int                 cgpr_planes;	/* color bit plane mask reg */
    struct pr_pos       cgpr_offset;	/* pixrect offset */
    short               cg2_index;	/* cg2 board index */
    char                minordev;	/* true minor dev to stuff into GP */
    int                 gbufflag;	/* gbuffer flag */
    int                 ioctl_fd;	/* the fd to talk to the driver with */
    int                 ncmd;		/* length of cmdver array */
    u_char             *cmdver;		/* version #'s for each command */
    int                 flags;		/* misc options */
    int                 linebytes;	/* bytes per line (pixel mode) */
    int                 fbtype;		/* which cg is bound */

    /* then make it look like a memory device with multiple plane groups */

    struct mprp_data    mprp;		/* memory pixrect simulator */
    int                 cg12_flags;	/* misc. flags */
    int                 planes;		/* current group and mask */
    int                 fd;		/* file descriptor */
    short               active;		/* active fb no. */

    /* finally get some cg12 specific portions in */

    struct cg12_ctl_sp *ctl_sp;
    struct cg12fb       fb[CG12_NFBS];	/* frame buffer info */
    struct fb_wid_dbl_info	wid_dbl_info;	/* window id */
    int			windowfd;
};

/* cg12 specific constants */

/* using shorts so at 4bits/pixel there are 4 pixels per short */

#define CG12_START_OFFSCREEN   ((CG12_WIDTH * CG12_HEIGHT) >> 2)

/* amount to offset from beginning of offscreen memory for dither data */
#define CG12_DITHER_BYTES       256
#define CG12_DITHER_PIXELS     (CG12_DITHER_BYTES << 1)
#define CG12_MAX_DITHER_DATA   (CG12_DITHER_PIXELS >> 2) 

/* amount to offset from beginning of offscreen memory for solid pattern data */
#define CG12_SOLID_PAT_BYTES    8
#define CG12_SOLID_PAT_PIXELS   (CG12_SOLID_PAT_BYTES << 1)
#define CG12_MAX_SOLID_PAT_DATA  (CG12_SOLID_PAT_PIXELS >> 2)

/* start loc for downloading bitmap data, padding needed? XXX*/

#define CG12_BITMAP_DATA_START  (CG12_START_OFFSCREEN +  CG12_MAX_DITHER_DATA + \
			    CG12_MAX_SOLID_PAT_DATA)

/* HACCESS value for 4 bits/pixel when loading offscreen VRAM */

#define CG12_OS_HACCESS 0x22

/* PLN_WR_MASK value for loading lower 4bits of each DPU offscreen VRAM */

#define CG12_OS_PLN_WR_MASK 0x000F0F0F

/*
/* useful macros */

#define	cg12_d(pr)	((struct cg12_data *) ((pr)->pr_data))

#define CG12_PR_TO_MEM(src, mem)					\
    if (src && src->pr_ops != &mem_ops)					\
    {									\
	(void) cg12_set_state(src);					\
	mem.pr_ops      = &mem_ops;					\
	mem.pr_size     = src->pr_size;					\
	mem.pr_depth    = src->pr_depth;				\
	mem.pr_data     = (char *) &cg12_d(src)->mprp;			\
	src             = &mem;						\
    }

extern struct pixrectops cg12_ops;

EXTERN_FUNCTION ( extern int cg12_ioctl,      (Pixrect*, int, caddr_t) );
EXTERN_FUNCTION ( extern int cg12_putcolormap,      (Pixrect*, int, int, unsigned char*, unsigned char*, unsigned char*) );
EXTERN_FUNCTION ( extern int cg12_putattributes,      (Pixrect*, u_int*) );
EXTERN_FUNCTION ( extern int cg12_rop,      (Pixrect*, int, int, int, int, int, Pixrect*, int, int) );

#ifndef KERNEL
EXTERN_FUNCTION ( extern int gp1_rop,       (Pixrect*, int, int, int, int, int, Pixrect*, int, int) );
EXTERN_FUNCTION ( extern int cg12_batchrop, (Pixrect*, int, int, int, struct pr_prpos*, int) );
EXTERN_FUNCTION ( extern int cg12_destroy,  (Pixrect*) );
EXTERN_FUNCTION ( extern int cg12_get,      (Pixrect*, int, int) );
EXTERN_FUNCTION ( extern int cg12_getattributes,      (Pixrect*, int*) );
EXTERN_FUNCTION ( extern int cg12_getcolormap,      (Pixrect*, int, int, unsigned char*, unsigned char*, unsigned char*) );
EXTERN_FUNCTION ( extern Pixrect *cg12_make,      (int, struct pr_size, int, struct fbgattr*) );
EXTERN_FUNCTION ( extern int cg12_put,      (Pixrect*, int, int, int) );
EXTERN_FUNCTION ( extern Pixrect *cg12_region,      (Pixrect*, int, int, int, int) );
EXTERN_FUNCTION ( extern int cg12_stencil,      (Pixrect*, int, int, int, int, int, Pixrect*, int, int, Pixrect*, int, int) );
EXTERN_FUNCTION ( extern int cg12_vector,      (Pixrect*, int, int, int, int, int, int) );


#endif	/* !KERNEL */

#endif	/* cg12var_DEFINED */
