/*	@(#)rectlist.h 10.4 88/02/08 SMI	*/

/*
 * Copyright (c) 1983 by Sun Microsystems, Inc.
 */

/*
 * Defines the interface to the data structure called 
 * a rectlist which is a list of rectangles.
 *
 * A rectlist has an offset (rl_x, rl_y) assoicated with it that is used
 * to efficiently pass this data structure from one embedded coordinate
 * system to another without changing the offsets of all the rectangles in
 * the list.
 *
 * Rl_bound is the rectangle that bounds all the rectangles in the list
 * and is maintained to facilitate efficient rectangle algebra.
 */

#ifndef _rectlist_defined
#define _rectlist_defined
typedef	struct	rectnode {
	struct	rectnode *rn_next;	/* Pointer to next rectnode */
	struct	rect rn_rect;
} Rectnode;

#define	RECTNODE_NULL	((Rectnode *)0)

typedef	struct	rectlist {
	coord	rl_x, rl_y;		/* Offset to apply to each rect
					   in list including bound */
	struct	rectnode *rl_head;	/* Pointer to first rectnode */
	struct	rectnode *rl_tail;	/* Pointer to last rectnode */
	struct	rect rl_bound;		/* Describes bounding rect of 
					   all rects in list */
} Rectlist;

#define	RECTLIST_NULL	((Rectlist *)0)

extern	struct rectlist rl_null;

/*
 * Rectlist Transformation macros used for passing rectlists up/down embedded
 * coordinate systems.
 */
#define	rl_passtoparent(x,y,rl) \
	{(rl)->rl_x=(rl)->rl_x+(x); (rl)->rl_y=(rl)->rl_y+(y);}

#define	rl_passtochild(x,y,rl) \
	{(rl)->rl_x=(rl)->rl_x-(x); (rl)->rl_y=(rl)->rl_y-(y);}

/*
 * Rectlist Offset Adjustment macros
 */
#define	rl_rectoffset(rl,r1,r) \
	{*(r)= *(r1); (r)->r_left+=(rl)->rl_x; (r)->r_top+=(rl)->rl_y;}

#define	rl_coordoffset(rl,x,y) {*(x)-=(rl)->rl_x;*(y)-=(rl)->rl_y;}


/* Was "#ifdef cplus" */
#if defined(__cplusplus) || defined(c_plusplus) || defined(__STDC__)
/* Declare extern function prototypes */

/*
 * C Library routine specifications related to rectlist functions.
 */

/*
 * Rectlist geometry predicates and functions
 */
EXTERN_FUNCTION(bool rl_empty, (Rectlist *) );
EXTERN_FUNCTION(bool rl_equal, (Rectlist *, Rectlist *) );
EXTERN_FUNCTION(void rl_intersection,
				(Rectlist *, Rectlist *, Rectlist *result) );
EXTERN_FUNCTION(void rl_sort, (Rectlist *, Rectlist *result, int order) );

EXTERN_FUNCTION(void rl_union, (Rectlist *, Rectlist *, Rectlist *result) );
EXTERN_FUNCTION(void rl_difference,
				(Rectlist *, Rectlist *, Rectlist *result) );
EXTERN_FUNCTION(void rl_coalesce, (Rectlist *) );
#ifndef KERNEL
EXTERN_FUNCTION(bool rl_includespoint, (Rectlist *, coord x, coord y) );
#endif !KERNEL

/*
 * Rectlist with Rect geometry predicates and functions
 */
#ifndef KERNEL
EXTERN_FUNCTION(bool rl_equalrect, (Rect *, Rectlist *) );
#endif !KERNEL
EXTERN_FUNCTION(bool rl_boundintersectsrect, (Rect *,  Rectlist *) );
EXTERN_FUNCTION(bool rl_rectintersects, (Rect *,  Rectlist *) );
EXTERN_FUNCTION(void rl_rectintersection,
				(Rect *, Rectlist *, Rectlist *result) );
EXTERN_FUNCTION(void rl_rectunion, (Rect *, Rectlist *, Rectlist *result) );
EXTERN_FUNCTION(void rl_rectdifference,
				(Rect *, Rectlist *, Rectlist *result) );

/*
 * Rectlist initialization functions
 */
EXTERN_FUNCTION(void rl_initwithrect, (Rect *, Rectlist *) );

/*
 * Rectlist List Memory Management functions
 */
EXTERN_FUNCTION(void rl_copy, (Rectlist *from, Rectlist *result) );
EXTERN_FUNCTION(void rl_free, (Rectlist *) );

/*
 * Rectlist Offset Adjustment functions
 */
EXTERN_FUNCTION(void rl_normalize, (Rectlist *) );

/*
 * Debugging Utilities
 */
EXTERN_FUNCTION(void rl_print, (Rectlist *, char *) );

#else
/* Not using ANSI C or C++:  don't declare extern function prototypes */
/* Just declare the return types of the few that were already here */

extern	bool rl_empty(), rl_equal(), rl_boundintersectsrect(),
		rl_rectintersects();

#ifndef KERNEL
extern	bool rl_equalrect(), rl_includespoint();
#endif !KERNEL

#endif
#endif /*!_rectlist_defined*/
