/*	@(#)win_struct.h 10.6 88/02/08 SMI	*/
#ifndef sunwindow_win_struct_DEFINED
#define sunwindow_win_struct_DEFINED

/*
 * Copyright (c) 1986 by Sun Microsystems, Inc.
 */

/*
 * This header file defines the library interface to most of
 * the kernel functions of the window system.  The remainder
 * functions are defined in pixwin.h.
 *
 * The kernel maintains a tree structure per-screen of
 * the relationship between the windows on the screen.
 * The top-level window in each screen is called the
 * root window and is generally owned by a screen manager.
 * Subwindows of the root window are generally owned by
 * ``tools'', which themselves may consist of multiple
 * windows at deeper levels.  The window manager deals
 * with the tools, moving the deeper levels en-masse.
 *
 * A window is created by calling win_getnewwindow().
 * An existing window can be opened multiple times by multiple processes and
 * the file descriptor is that which is used as the "ticket" to change
 * something about a window.
 *
 * Each window tracks the cursor with a shape...
 *
 * Each window has a parent, a place within a set of siblings
 * (specified as the oldest and youngest siblings), and a
 * set of children (specified as a oldest and youngest child).
 * These links are maintained by the system, and checked for
 * consistency when windows are inserted into and deleted from
 * the window tree.
 *
 * Each window competes for input.  Whenever keyboard or mouse
 * actions occur, the system gives the window in which the cursor
 * is located a chance at the input.  If that window is not
 * interested in the input, then its parent is given a chance,
 * and so on.  If no one is interested, the input is discarded.
 * Input from a window may cause interrupts to be sent to the
 * ``owner process'' for the window.
 *
 * Each window also maintains a ``clipping list'' and a ``damaged list''.
 * The clipping list gives the set of rectangles which are currently
 * exposed for the window.  The damaged list gives the set of
 * rectangles which are exposed, but known to be damaged by
 * recent window rearrangements, and which should therefore be
 * repainted by the responsible processes.
 */

#define	WIN_NAMESIZE	20

/*
 * Link names.
 */
#define	WL_PARENT		0
#define	WL_OLDERSIB		1
#define	WL_YOUNGERSIB		2
#define	WL_OLDESTCHILD		3
#define	WL_YOUNGESTCHILD	4

#define	WL_ENCLOSING		WL_PARENT
#define	WL_COVERED		WL_OLDERSIB
#define	WL_COVERING		WL_YOUNGERSIB
#define	WL_BOTTOMCHILD		WL_OLDESTCHILD
#define	WL_TOPCHILD		WL_YOUNGESTCHILD

#define	WIN_LINKS		5
#define	WIN_NULLLINK		-1

/*
 * Flag to use during open of window device when it is supposed to be the
 * first time opened (exclusive open).  O_EXCL in file.h currently has wrong
 * mapping to FEXLOCK.  Need a FEXCLOPEN in file.h.
 */
#define	WIN_EXCLOPEN		0x10000

#if defined(__cplusplus) || defined(c_plusplus) || defined(__STDC__)

#ifndef C_VARIETIES_H
#include <c_varieties.h>
#endif
/* Declare extern function prototypes */

/*
 * C Library routine specifications relating
 * to kernel supported window functions.
 */

/*
 * Error handling (overriding library error handler, returns previous handler)
 */
typedef int Win_error_proc(int errnum, int winopnum);
EXTERN_FUNCTION(Win_error_proc *win_errorhandler, (Win_error_proc*) );

/*
 * Tree operations.
 */
EXTERN_FUNCTION(int	win_getlink, (int fd, int linkname) );
EXTERN_FUNCTION(void	win_setlink, (int fd, int linkname, int number) );
EXTERN_FUNCTION(void	win_insert, (int fd) );
EXTERN_FUNCTION(void	win_remove, (int fd) );
EXTERN_FUNCTION(int	win_nextfree, (int fd) );

/* utilities */
EXTERN_FUNCTION(void	win_numbertoname, (int number, char *name) );
EXTERN_FUNCTION(int	win_nametonumber, (char *name) );
EXTERN_FUNCTION(void	win_fdtoname, (int fd, char *name) );
EXTERN_FUNCTION(int	win_fdtonumber, (int fd) );
EXTERN_FUNCTION(int	win_getnewwindow, (_VOID_) );

/*
 * Mouse cursor operations.
 */
EXTERN_FUNCTION(void	win_setmouseposition, (int fd, short x, short y) );
EXTERN_FUNCTION(void	win_setcursor, (int fd, struct cursor *cursor) );
EXTERN_FUNCTION(void	win_getcursor, (int fd, struct cursor *cursor) );
EXTERN_FUNCTION(int	win_findintersect, (int fd, short x, short y) );

/*
 * Geometry operations.
 */
EXTERN_FUNCTION(void	win_getrect, (int fd, struct rect *rect) );
EXTERN_FUNCTION(void	win_setrect, (int fd, struct rect *rect) );
EXTERN_FUNCTION(void	win_setsavedrect, (int fd, struct rect *rect) );
EXTERN_FUNCTION(void	win_getsavedrect, (int fd, struct rect *rect) );

/* utilities */
EXTERN_FUNCTION(void	win_getsize, (int fd, struct rect *rect) );
EXTERN_FUNCTION(coord	win_getheight, (int fd) );
EXTERN_FUNCTION(coord	win_getwidth, (int fd) );

/* blanket window operations */
EXTERN_FUNCTION(int	win_insertblanket, (int windowfd, int parentfd) );
EXTERN_FUNCTION(void	win_removeblanket, (int fd) );
EXTERN_FUNCTION(int	win_isblanket, (int fd) );

/*
 * Misc operations.
 */
EXTERN_FUNCTION(int	win_getuserflags, (int fd) );
EXTERN_FUNCTION(void	win_setuserflags, (int fd, int flags) );
EXTERN_FUNCTION(void	win_setowner, (int fd, int pid) );
EXTERN_FUNCTION(int	win_getowner, (int fd) );

/* utilities */
EXTERN_FUNCTION(void	win_setuserflag, (int fd, int flag, bool value) );

/*
 * Input control.
 */
EXTERN_FUNCTION(void	win_getinputmask, (int fd,
				struct inputmask *,
				int *inputredirectwindownumber) );
EXTERN_FUNCTION(void	win_setinputmask, (int fd,
				struct inputmask *im_set,
				struct inputmask *im_flush,
				int inputredirectwindownumber) );

/*
 * Kernel operations applying globally to a screen.
 */
EXTERN_FUNCTION(void	win_lockdata, (int fd) );
EXTERN_FUNCTION(void	win_computeclipping, (int fd) );
EXTERN_FUNCTION(void	win_partialrepair, (int fd, struct rect *) );
EXTERN_FUNCTION(void	win_unlockdata, (int fd) );
EXTERN_FUNCTION(void	win_grabio, (int fd) );
EXTERN_FUNCTION(void	win_releaseio, (int fd) );

/*
 * Display management operations (see pixwin.h).
 */

/*
 * Screen creation, inquiry and deletion.
 */
EXTERN_FUNCTION(int	win_screennew, (struct screen *) );
EXTERN_FUNCTION(void	win_screenget, (int fd, struct screen *) );
EXTERN_FUNCTION(void	win_screendestroy, (int fd) );
EXTERN_FUNCTION(void	win_setscreenpositions, (int fd, DOTDOTDOT) );
EXTERN_FUNCTION(void	win_getscreenpositions, (int fd, DOTDOTDOT) );
#endif /* "cplus" */

#endif sunwindow_win_struct_DEFINED

