/*	@(#)time.h 2.10 88/02/08 SMI; from UCB 7.1 6/4/86	*/

/*
 * Copyright (c) 1982, 1986 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 */

#ifndef __sys_time_h
#define __sys_time_h

#ifndef C_VARIETIES_H
#include <c_varieties.h>
#endif

#include <sys/types.h>

/*
 * Structure returned by gettimeofday(2) system call,
 * and used in other calls.
 */
struct timeval {
	long	tv_sec;		/* seconds */
	long	tv_usec;	/* and microseconds */
};

struct timezone {
	int	tz_minuteswest;	/* minutes west of Greenwich */
	int	tz_dsttime;	/* type of dst correction */
};
#define	DST_NONE	0	/* not on dst */
#define	DST_USA		1	/* USA style dst */
#define	DST_AUST	2	/* Australian style dst */
#define	DST_WET		3	/* Western European dst */
#define	DST_MET		4	/* Middle European dst */
#define	DST_EET		5	/* Eastern European dst */
#define	DST_CAN		6	/* Canada */
#define	DST_GB		7	/* Great Britain and Eire */
#define	DST_RUM		8	/* Rumania */
#define	DST_TUR		9	/* Turkey */
#define	DST_AUSTALT	10	/* Australian style with shift in 1986 */

/*
 * Operations on timevals.
 *
 * NB: timercmp does not work for >= or <=.
 */
#define	timerisset(tvp)		((tvp)->tv_sec || (tvp)->tv_usec)
#define	timercmp(tvp, uvp, cmp)	\
	((tvp)->tv_sec cmp (uvp)->tv_sec || \
	 (tvp)->tv_sec == (uvp)->tv_sec && (tvp)->tv_usec cmp (uvp)->tv_usec)
#define	timerclear(tvp)		(tvp)->tv_sec = (tvp)->tv_usec = 0

/*
 * Names of the interval timers, and structure
 * defining a timer setting.
 */
#define	ITIMER_REAL	0
#define	ITIMER_VIRTUAL	1
#define	ITIMER_PROF	2

struct	itimerval {
	struct	timeval it_interval;	/* timer interval */
	struct	timeval it_value;	/* current value */
};

#ifndef KERNEL
#include <time.h>
#endif

#ifdef SYS_adjtime
EXTERN_FUNCTION ( int adjtime,		(const timeval*, timeval*) );
#endif
EXTERN_FUNCTION ( int getitimer,	(int, itimerval*) );
EXTERN_FUNCTION ( int setitimer,	(int, const itimerval*, itimerval*) );
EXTERN_FUNCTION ( int gettimeofday,	(timeval*, struct timezone*) );
EXTERN_FUNCTION ( int settimeofday,	(const timeval*, const struct timezone*) );
EXTERN_FUNCTION ( int utimes,		(const char*, timeval tvp[2]) );
EXTERN_FUNCTION ( int vtimes,		(struct vtimes*, struct vtimes*));
EXTERN_FUNCTION ( int select,		(int, fd_set*, fd_set*, fd_set*, const struct timeval*) );
EXTERN_FUNCTION ( extern int rtime,     (struct sockaddr_in*, struct timeval*, struct timeval*)); 
EXTERN_FUNCTION ( extern int aiowait,   (struct timeval *)); 

#endif /* _sys_time_h */ 
